```@meta
CurrentModule = Oscar
```

# Matroid Realization Spaces


Let ``M`` be a matroid of rank ``d`` on a ground set ``E`` of size ``n``. Its
*realization space* ``\mathcal{R}(M)`` is an affine scheme that parameterizes
all hyperplane arrangements that realize the matroid ``M`` (up to the action of
``PGL(r)``).  We provide functions that determine the affine coordinate ring of
``\mathcal{R}(M)``. 


```@docs
is_realizable(M::Matroid; char::Union{Int,Nothing}=nothing, q::Union{Int,Nothing}=nothing)
defining_ideal(RS::MatroidRealizationSpace)
inequations(RS::MatroidRealizationSpace)
ambient_ring(RS::MatroidRealizationSpace)
realization_space(M::Matroid; B::Union{GroundsetType,Nothing} = nothing, 
    F::AbstractAlgebra.Ring = ZZ, saturate::Bool=false, reduce::Bool=true,
    char::Union{Int,Nothing}=nothing, q::Union{Int,Nothing}=nothing)
realization(M::Matroid; B::Union{GroundsetType,Nothing} = nothing, 
    F::AbstractAlgebra.Ring = ZZ, saturate::Bool=false, reduce::Bool=true,
    char::Union{Int,Nothing}=nothing, q::Union{Int,Nothing}=nothing)
realization(RS::MatroidRealizationSpace)
```

If ``B`` is the polynomial ring `ambient_ring(RS)`, ``I`` the ideal
`defining_ideal(RS)`, and ``U`` the multiplicative semigroup generated by
`inequations(RS)`, then the coordinate ring of the realization space
``\mathcal{R}(M)`` is isomorphic to ``U^{-1}B/I``.  


